export declare class Right<T> {
    readonly _value: T;
    private _tag;
    constructor(_value: T);
}
export declare class Left<E> {
    readonly _error: E;
    private _tag;
    constructor(_error: E);
}
export declare type Either<T, E> = Right<T> | Left<E>;
export declare const right: <T>(value: T) => Right<T>;
export declare const left: <E>(error: E) => Left<E>;
export declare const isRight: <T, E>(value: Either<T, E>) => boolean;
export declare const isLeft: <T, E>(value: Either<T, E>) => boolean;
export declare const withDefault: <T, E>(value: Either<T, E>, defaultValue: T) => T;
export declare const map: <A, B, E = Error>(f: (a: A) => B, value: Either<A, E>) => Either<B, E>;
export declare const andThen: <A, B, E = Error>(f: (a: A) => Either<B, E>, value: Either<A, E>) => Either<B, E>;
export declare const caseOf: <A, B, E = Error>(caseof: {
    Right: (v: A) => B;
    Left: (v: E) => B;
}, value: Either<A, E>) => B;
