"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Right = /** @class */ (function () {
    function Right(_value) {
        this._value = _value;
        this._tag = 'right';
    }
    return Right;
}());
exports.Right = Right;
var Left = /** @class */ (function () {
    function Left(_error) {
        this._error = _error;
        this._tag = 'left';
    }
    return Left;
}());
exports.Left = Left;
var assertIsEither = function (value) {
    if (!(value instanceof Right || value instanceof Left)) {
        throw new Error("Value \"" + value + "\" is not an Either type");
    }
};
exports.right = function (value) {
    return new Right(value);
};
exports.left = function (error) {
    return new Left(error);
};
exports.isRight = function (value) {
    assertIsEither(value);
    return value instanceof Right;
};
exports.isLeft = function (value) {
    assertIsEither(value);
    return value instanceof Left;
};
exports.withDefault = function (value, defaultValue) {
    switch (exports.isLeft(value)) {
        case true:
            return defaultValue;
        case false:
            return value._value;
    }
};
exports.map = function (f, value) {
    switch (exports.isLeft(value)) {
        case true:
            return value;
        case false:
            try {
                return exports.right(f(value._value));
            }
            catch (error) {
                return exports.left(error);
            }
    }
};
exports.andThen = function (f, value) {
    switch (exports.isLeft(value)) {
        case true:
            return value;
        case false:
            return f(value._value);
    }
};
exports.caseOf = function (caseof, value) {
    switch (exports.isLeft(value)) {
        case true:
            return caseof.Left(value._error);
        case false:
            return caseof.Right(value._value);
    }
};
